import { microMTAErrorEventListener, microMTAMessageEventListener, microMTARejectedEventListener } from './events';
import { microMTAOptions } from './options';
export declare class microMTA {
    private server;
    private tlsServer;
    private connections;
    private events;
    private options;
    constructor(options?: microMTAOptions);
    close(): void;
    /**
     * Adds a listener for a rejected message event.
     * @param eventType Event type. (rejected)
     * @param listener Listener function.
     */
    on(eventType: 'rejected', listener: microMTARejectedEventListener): void;
    /**
     * Adds a listener for a message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    on(eventType: 'message', listener: microMTAMessageEventListener): void;
    /**
     * Adds a listener for an error event.
     * @param eventType Event type. (error)
     * @param listener Listener function.
     */
    on(eventType: 'error', listener: microMTAErrorEventListener): void;
    /**
     * Removes a listener for a message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    off(eventType: 'rejected', listener: microMTARejectedEventListener): void;
    /**
     * Removes a listener for a message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    off(eventType: 'message', listener: microMTAMessageEventListener): void;
    /**
     * Removes a listener for an error event.
     * @param eventType Event type. (error)
     * @param listener Listener function.
     */
    off(eventType: 'error', listener: microMTAErrorEventListener): void;
    private emit;
    private connection;
    private pruneConnections;
}
