"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.microMTA = void 0;
var net_1 = require("net");
var tls_1 = require("tls");
var connection_1 = require("./connection");
var replies_1 = require("./smtp/replies");
var microMTA = /** @class */ (function () {
    function microMTA(options) {
        var _this = this;
        this.connections = new Set();
        this.events = {
            message: new Set(),
            error: new Set(),
            rejected: new Set(),
        };
        this.options = {
            hostname: 'localhost',
            ip: '0.0.0.0',
            port: 25,
            tlsPort: 465,
            size: 10000000,
        };
        this.options = __assign(__assign({}, this.options), options);
        this.server = net_1.createServer(function (socket) { return _this.connection(socket); });
        this.server.listen(this.options.port, this.options.ip);
        if (this.options.tlsPort && this.options.secureContextOptions) {
            this.tlsServer = tls_1.createServer({
                secureContext: tls_1.createSecureContext(this.options.secureContextOptions),
            }, function (socket) { return _this.connection(socket); });
            this.tlsServer.listen(this.options.tlsPort, this.options.ip);
        }
        setInterval(this.pruneConnections.bind(this), 1000);
    }
    microMTA.prototype.close = function () {
        var _this = this;
        this.connections.forEach(function (connection) {
            if (connection.isOpen) {
                connection.reply.apply(connection, __spread(replies_1.SERVER_SHUTTING_DOWN));
                connection.close();
            }
        });
        this.server.close(function () { return _this.server.unref(); });
    };
    /**
     * Adds a listener for a given event.
     * @param eventType Event type.
     * @param listener Listener function.
     */
    microMTA.prototype.on = function (eventType, listener) {
        this.events[eventType].add(listener);
    };
    /**
     * Removes a listener for a given event.
     * @param eventType Event type.
     * @param listener Listener function.
     */
    microMTA.prototype.off = function (eventType, listener) {
        this.events[eventType].delete(listener);
    };
    microMTA.prototype.emit = function (eventType) {
        var e_1, _a;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        try {
            for (var _b = __values(this.events[eventType]), _c = _b.next(); !_c.done; _c = _b.next()) {
                var listener = _c.value;
                listener.apply(this, args);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    microMTA.prototype.connection = function (socket) {
        var _this = this;
        this.connections.add(new connection_1.microMTAConnection(socket, this.options, function (message) { return _this.emit('message', message); }, function (error) { return _this.emit('error', error); }, function (sender, recipients) { return _this.emit('rejected', sender, recipients); }));
    };
    microMTA.prototype.pruneConnections = function () {
        var _this = this;
        this.connections.forEach(function (connection) {
            if (!connection.isOpen) {
                _this.connections.delete(connection);
            }
        });
    };
    return microMTA;
}());
exports.microMTA = microMTA;
